﻿using System;
using System.Globalization;
using System.ServiceModel;
using System.Threading;
using RockyJupiter.Mobilengine.Client.Wdx;

namespace RockyJupiter.Mobilengine.Client
{
    class Program
    {
        //Inbound endpoint key
        //in a real-world scenario this would come from configuration
        private const string Key = "333da03781b04fbbb78ad991d4986c78";

        private const int MaxRetryCount = 5;
        private static readonly TimeSpan RetryInterval = TimeSpan.FromSeconds(10);

        static void Main(string[] args)
        { 
            var retryCount = 0;
            bool shouldRetry;
            do
            {
                var wdxClient = new WdxClient();
                try
                {
                    retryCount++;
                    wdxClient.EnqueueDacs(CreateDacs());
                    wdxClient.Close();
                    Console.WriteLine("EnqueueDacs succeeded.");
                    shouldRetry = false;
                }
                catch (FaultException<EnqueueDacsFail>)
                {
                    Console.WriteLine("EnqueueDacsFail fault received, giving up.");
                    shouldRetry = false;
                }
                catch (Exception ex)
                {
                    wdxClient.Abort();
                    shouldRetry = retryCount < MaxRetryCount;
                    Console.WriteLine("EnqueueDacs failed: {0}", ex.Message);
                    if (shouldRetry)
                    {
                        Console.WriteLine("Will retry in {0} seconds.", RetryInterval.TotalSeconds);
                        Thread.Sleep(RetryInterval);
                    }
                    else
                    {
                        Console.WriteLine("Giving up after {0} retries", retryCount);
                    }
                }
            } while (shouldRetry);
        }

        static Dacs CreateDacs()
        {
            return new Dacs
            {
                dacsid = Guid.NewGuid().ToString("N"),
                dtu = DateTime.UtcNow,
                Key = Key,
                meta = DacsMeta.rockyJupiternewPO,
                Content = new DacsContent
                {
                    Item = CreateNewPo()
                }
            };
        }

        static newPO CreateNewPo()
        {
            return new newPO
            {
                assignmentId = Guid.NewGuid().ToString("N"),
                admin = "Connie Lowe",
                driver = "petar.hoyt@gmail.com",
                client = "Sonky Rubber Goods",
                loadAddress = "Napierstraat 33 1972 EE IJmuiden",
                unloadAddress = "Edisonstraat 57 5615 GA Eindhoven",
                dueDate = (DateTime.UtcNow + TimeSpan.FromDays(5)).ToString("MM/dd/yyyy HH:mm:ss.fffK", CultureInfo.InvariantCulture),
                loadType = NewPOLoadType.Shippingcontainer,
            };
        }
    }
}
